<?php

namespace MattChoinski\EmployeeDatabase\Model
{
	require_once( realpath( $_SERVER[ "DOCUMENT_ROOT" ] ) . "/helper/common.php" );

	class Employee extends Base
	{
		private static $dataQuery = <<<DataQuery
SELECT a.Id
      ,a.DateCreated
      ,a.Email
      ,a.IsApproved
      ,a.ManagerId
      ,a.NameFirst
      ,a.NameLast
      ,a.Password
      ,a.RoleId
      ,b.Email AS ManagerEmail
      ,b.NameFirst AS ManagerNameFirst
      ,b.NameLast AS ManagerNameLast
  FROM Employee AS a
  LEFT OUTER JOIN Employee AS b
    ON b.Id = a.ManagerId
DataQuery;

		public $Email;
		public $IsApproved;
		public $ManagerEmail;
		public $ManagerId;
		public $ManagerNameFirst;
		public $ManagerNameLast;
		public $NameFirst;
		public $NameLast;
		public $Password;
		public $RoleId;


		public static function FetchAll()
		{
			$dataCriteria = <<<DataCriteria
  WHERE a.IsApproved = 1;
DataCriteria;

			return
				self::Fetch(
					self::$dataQuery
					. $dataCriteria );
		}

		public static function FetchByEmail(
			$email )
		{
			$result = null;

			$dataCriteria = <<<DataCriteria
 WHERE a.Email = '$email';
DataCriteria;
			$modelCollection =
				self::Fetch(
					self::$dataQuery
					. $dataCriteria );
			if ( !empty( $modelCollection[ 0 ] ) )
			{
				$result = $modelCollection[ 0 ];
			}
;
			return $result;
		}

		public static function FetchById(
			$id = null )
		{
			$result = null;

			if ( !isset( $id ) )
			{
				if ( isset( $_GET[ "id" ] ) )
				{
					$id = $_GET[ "id" ];
				}
				else
				{
					$id = \MattChoinski\EmployeeDatabase\Helper\Session::Fetch( "id" );
				}
			}
			$dataCriteria = <<<DataCriteria
 WHERE a.Id = $id;
DataCriteria;
			$model =
				self::Fetch(
					self::$dataQuery
					. $dataCriteria );
			if ( !empty( $model[ 0 ] ) )
			{
				$result = $model[ 0 ];
			}

			return $result;
		}

		public static function FetchManager()
		{
			$dataCriteria = <<<DataCriteria
  WHERE a.Id > 1
    AND a.ManagerId IS NULL
     OR a.ManagerId = 0;
DataCriteria;

			return
				self::Fetch(
					self::$dataQuery
					. $dataCriteria );
		}

		public static function FetchNonApproved()
		{
			$dataCriteria = <<<DataCriteria
  WHERE a.IsApproved = 0;
DataCriteria;

			return
				self::Fetch(
					self::$dataQuery
					. $dataCriteria );
		}

		public static function FetchTeam(
			$id = null )
		{
			if ( !isset( $id ) )
			{
				$id = \MattChoinski\EmployeeDatabase\Helper\Session::Fetch( "id" );
			}
			$dataCriteria = <<<DataCriteria
  WHERE a.ManagerId = {$id};
DataCriteria;

			return
				self::Fetch(
					self::$dataQuery
					. $dataCriteria );
		}

		public static function HasSession()
		{
			$result = false;

			$userAgent = \MattChoinski\EmployeeDatabase\Helper\Session::Fetch( "user_agent" );
			if ( isset( $userAgent ) )
			{
				if ( $userAgent == md5( $_SERVER[ "HTTP_USER_AGENT" ] ) )
				{
					$result = true;
				}
			}

			return $result;
		}

		public static function Insert(
			$model )
		{
			$model->ManagerId = 0;
			$model->Password =
				crypt(
					"password.1",
					SALT );
			$model->RoleId = 2;

			$dataQuery = <<<DataQuery
INSERT
  INTO Employee
       (
         Email
        ,ManagerId
        ,NameFirst
        ,NameLast
        ,Password
        ,RoleId
       )
VALUES
       (
         '{$this->Email}'
        ,{$this->ManagerId}
        ,'{$this->NameFirst}'
        ,'{$this->NameLast}'
        ,'{$this->Password}'
        ,{$this->RoleId}
       );
DataQuery;

			return parent::Execute( $dataQuery );
		}

		public static function Set()
		{
			$model = new Employee();
 			$model->Id =
 				filter_input(
 					INPUT_POST,
 					"id",
 					FILTER_SANITIZE_NUMBER_INT );
 			$model->Email =
 				filter_input(
 					INPUT_POST,
 					"email",
 					FILTER_SANITIZE_STRING );
 			$model->IsApproved =
 				filter_input(
 					INPUT_POST,
 					"is_approved",
 					FILTER_SANITIZE_NUMBER_INT );
 			$model->ManagerId =
 				filter_input(
 					INPUT_POST,
 					"manager_id",
 					FILTER_SANITIZE_NUMBER_INT );
 			$model->NameFirst =
 				filter_input(
 					INPUT_POST,
 					"name_first",
 					FILTER_SANITIZE_STRING );
 			$model->NameLast =
 				filter_input(
 					INPUT_POST,
 					"name_last",
 					FILTER_SANITIZE_STRING );
 			$model->Password =
 				filter_input(
 					INPUT_POST,
 					"password",
 					FILTER_SANITIZE_STRING );
			if ( !empty( $model->Password ) )
			{
				$bcrypt = new \MattChoinski\EmployeeDatabase\Helper\Bcrypt();
				$model->Password = $bcrypt->hash( $model->Password );
			}
 			$model->RoleId =
 				filter_input(
 					INPUT_POST,
 					"role_id",
 					FILTER_SANITIZE_NUMBER_INT );

			if ( $model->Id == 0 )
			{
				$model->Id = self::Insert( $model );
			}
			else
			{
				self::Update( $model );
			}
		}

		public static function Update(
			$model )
		{
			$createdById = \MattChoinski\EmployeeDatabase\Helper\Session::Fetch( "id" );
			$dataQuery = <<<DataQuery
UPDATE Employee
   SET CreatedById = {$createdById}
      ,Email = '{$model->Email}'
      ,ManagerId = {$model->ManagerId}
      ,NameFirst = '{$model->NameFirst}'
      ,NameLast = '{$model->NameLast}'
DataQuery;

			if ( !empty( $model->IsApproved ) )
			{
				$dataQuery =
					$dataQuery
					. <<<DataQuery
      ,IsApproved = {$model->IsApproved}
DataQuery;
			}

			if ( !empty( $model->Password ) )
			{
				$dataQuery =
					$dataQuery
					. <<<DataQuery
      ,Password = '{$model->Password}'
DataQuery;
			}

			if ( $model->RoleId > 0 )
			{
				$dataQuery =
					$dataQuery
					. <<<DataQuery
      ,RoleId = '{$model->RoleId}'
DataQuery;
			}

			$dataQuery =
				$dataQuery
				. <<<DataQuery
 WHERE Id = {$model->Id};
DataQuery;

			parent::Execute( $dataQuery );
		}
	}
}
